/**
 * Exposes parsing functionality that attempts to "upgrade" the parameters defined against an sp-list macro from a
 * previous version of the SharePoint Connector (over the course of the 1.x line, the parameter names and nuances have
 * changed quite a bit).
 *
 * In this way, the user should hopefully have a seamless experience when they load an sp-list macro with out-dated
 * parameters into the macro browser.
 *
 * There are some edge cases that probably won't work, but the sp-list macro itself *also* has in-depth, backwards-compatible
 * parameter parsing in its renderer, so even if the user has to re-type stuff into the macro browser in order to get it
 * to work, the macros shouldn't suddenly start breaking in the page view mode.
 */
SharePointConnector = {};

(function() {

    var knownValidParameterNames = ["site", "list", "listname", "folder", "view", "columns"];

    /**
     * In SharePoint Connector 1.4, the parameters for the sp-list macro were changed to be capitalised (eg. from
     * "list" to "List") so that they would look nicer in the macro browser. Unfortunately parameter names in the macro
     * browser are case-sensitive, so loading a pre-existing macro with old lower-case parameter names would not
     * populate the macro browser with the pre-existing parameter values. This function cludges the parameter map and
     * forces all parameters to be capitalised correctly, if they are 'known' as 'valid' parameters to the sp-list
     * macro.
     *
     * @param params Parameter Map supplied by AJS.MacroBrowser in the form params[name] = value.
     */
    var parameterCaseParser = function(params, log) {
        log("Entering sp-list parameter case parser");
        var toDelete = [];
        for (var parameterName in params) {
            if (!isKnownParameterName(parameterName)) {
                continue; // don't fudge with parameters that we don't recognise.
            }

            var fixedParameterName = capitalize(parameterName);
            if (fixedParameterName == parameterName) {
                continue; // Only do the switcheroo if the name has actually changed now that it's been capitalized.
            }

            // replace the old one with the new one.
            log("Replacing " + parameterName + " with " + fixedParameterName);
            params[fixedParameterName] = params[parameterName];
            params[parameterName] = null;
            toDelete.push(parameterName);
        }

        for (var paramToDelete in toDelete)
        {
            delete params[paramToDelete];
        }

        return params;
    };

    /**
     * In SharePoint Connector 1.4, the "ListName" parameter was renamed to just "List". Automatically rename this
     * parameter on-the-fly as it comes in to the macro browser.
     *
     * @param params Parameter Map supplied by AJS.MacroBrowser in the form params[name] = value.
     */
    var renameListNameToListParser = function(params, log) {
        log("Entering sp-list rename listname to list parser");
        var listNameValue = getParam(params, "ListName");
        if (listNameValue != null) {
            // rename "ListName" to "List".
            log("Renaming ListName to List");
            params["List"] = listNameValue;
            params["ListName"] = null;
            delete params["ListName"];
        }

        return params;
    };

    /**
     * In the early releases of the SharePoint Connector, the sp-list macro accepted two un-named parameters as input.
     * One for the site/list/folder combo, and another to define the columns. The macro browser does not play well
     * with un-named parameters, and this is a fairly non-standard behaviour for a Confluence macro, compared to every
     * other f$#%ing Confluence macro in existence.
     *
     * In SharePoint Connector 1.2 & 1.3, the parameters where changed to have explicit names, however we should still
     * make a best effort attempt to try and kludge old-school parameters from an existing version of the sp-list macro
     * into the macro browser dialog.
     *
     * Due to some problems with the Macro Browser's handling of un-named params (see CONF-19111), we're not going to
     * be able to achieve perfect backwards compatibility, but hopefully it will be good enough for most.
     *
     * @param params Parameter Map supplied by AJS.MacroBrowser in the form params[name] = value.
     */
    var unNamedParameterParser = function(params, log) {
        log("Entering sp-list un-named parameter parser");
        // In wiki-markup, the first un-named parameter found by the macro browser will be added to the params map
        // with a key of "". For us, this first un-named parameter *should* be the site:subsite/list//folder combo.
        // In rich-text mode (pre 4.0), this parameter is unfortunately missed from the map entirely and there's
        // nothing we can do.
        var unNamedParameter = getParam(params, "");
        if (unNamedParameter != null) {
            // replace the "List" parameter with this value.
            log("setting un-named parameter " + unNamedParameter + " as the list value.");
            params["List"] = unNamedParameter;
            params[""] = null;
            delete params[""];
        }

        // The second un-named parameter found by the macro browser will be added to the params map as if it were
        // a value-less keyed parameter. The most likely scenario here is that the second un-named parameter is the
        // "columns" formatting parameter. If so, we'll move it over into the columns param.
        for (var paramName in params) {
            // Look for any param that is not a known valid parameter for this macro. If it also has no value, then
            // assume it is the second un-named parameter.
            if (isKnownParameterName(paramName)) {
                continue;
            }

            var paramValue = params[paramName];
            if (paramValue != undefined && paramValue != null && paramValue != "") {
                continue;
            }

            log("parameter " + paramName + " is becoming the columns parameter.");
            params["Columns"] = paramName;
            delete params[paramName];
        }

        return params;
    };

    /**
     * In SharePoint Connector 1.3, the "Site" parameter was added. Previously, the site alias was embedded in the "list"
     * parameter in the form list=[site]:[list]. This parser attempts to retrieve the site parameter using the old
     * format.
     *
     * @param params Parameter Map supplied by AJS.MacroBrowser in the form params[name] = value.
     */
    var siteSplitterParser = function(params, log) {
        log("Entering sp-list site splitter parser.");
        // If the "Site" parameter is not specified, look for it in the "List" parameter.
        var siteParam = getParam(params, "Site");
        if (siteParam != null)
            return params;

        // Site parameter not defined. This could mean that the default site should be used, or the site parameter
        // is embedded in the list parameter using the old syntax of list=[site]:[list].
        var listParam = getParam(params, "List");
        if (listParam == null) {
            log("List parameter was undefined, even though it is a required parameter");
            return params;
        }

        var colonPos = listParam.indexOf(":");
        if (colonPos > 0) {
            siteParam = listParam.substring(0, colonPos);
            listParam = listParam.substring(colonPos + 1);

            log("Setting site:" + siteParam + " and list:" + listParam);
            params["List"] = listParam;
            params["Site"] = siteParam;
        }
        return params;
    };

    /**
     * In SharePoint Connector 1.4, the "Folder" parameter was added. Previously, the sub-folder was embedded in the "list"
     * parameter in the form list=[site]:[list]//[folder]. This parser attempts to retrieve the folder parameter using
     * the old format.
     *
     * @param params Parameter Map supplied by AJS.MacroBrowser in the form params[name] = value.
     */
    var folderSplitterParser = function(params, log) {
        log("Entering sp-list folder splitter parser.");
        // If the "Folder" parameter is not specified, look for it in the "List" parameter.
        var folderParam = getParam(params, "Folder");
        if (folderParam != null)
            return params;

        var listParam = params["List"];
        if (listParam == undefined) {
            log("List parameter was undefined, even though it is a required parameter");
            return params;
        }

        var slashPos = listParam.indexOf("//");
        if (slashPos > 0) {
            folderParam = listParam.substring(slashPos + 2);
            listParam = listParam.substring(0, slashPos);

            log("Setting list:" + listParam + " and folder:" + folderParam);
            params["List"] = listParam;
            params["Folder"] = folderParam;
        }

        return params;
    };

    /**
     *
     * @param params
     */
    SharePointConnector.parseParams = function(params, log) {
        // NOTE: The order in which these parsers run is important. For example, all the parsers assume that the
        // parameterCaseParser has already run and set the parameter names to the correct case.
        var chain = [parameterCaseParser, renameListNameToListParser, unNamedParameterParser, siteSplitterParser, folderSplitterParser];
        for (var i = 0; i < chain.length; i++) {
            var parser = chain[i];
            params = parser(params, log);
        }

        return params;
    };

    /**
     * Retrieves the parameter with the specified name. Handles edge cases where the parameter value is null,
     * un-defined, empty or whitespace and just returns null in all those cases.
     */
    var getParam = function(params, parameterName)
    {
        var param = params[parameterName];
        if (param == undefined || param == null || param == "")
            return null;

        // trim leading and trailing white-space.
        param = param.replace(/^\s*(\S*(?:\s+\S+)*)\s*$/, "$1");
        if (param == "")
            return null;

        return param;
    };

    /**
     * private helper function - ensures that the first character of the specified parameter name is upper case,
     * for some definition of "upper".
     *
     * @param parameterName the parameter name, duh.
     */
    var capitalize = function(parameterName) {
        return parameterName.substring(0, 1).toUpperCase() + parameterName.substring(1);
    };

    /**
     * private helper function - returns true if the specified parameter name is known to be a valid parameter to the
     * sp-list macro.
     *
     * @param parameterName the parameter name, duh.
     */
    var isKnownParameterName = function(parameterName) {
        for (var i = 0; i < knownValidParameterNames.length; i++)
        {
            var knownParameterName = knownValidParameterNames[i];
            if (parameterName.toLowerCase() == knownParameterName) {
                return true;
            }
        }
        return false;
    };
})();

